﻿Function Get-Inventory {

    [CmdletBinding()]
    Param
    (   [Parameter(HelpMessage="Computer or computers to gather information from",
                   ValueFromPipeline=$true,
                   ValueFromPipelineByPropertyName=$true,
                   Position=0)]
        $CompListPath='D:\Online.txt',
       
        [Parameter(HelpMessage="Maximum number of concurrent threads")]
        [ValidateRange(1,65535)]
        [int32]$ThrottleLimit = 16,

        [Parameter(HelpMessage="Display progress of function")]
        [switch]$ShowProgress
        
    )

    BEGIN  {
        $ofs = ';  '
        $runspacetimers       = [HashTable]::Synchronized(@{})
        $runspaces            = New-Object -TypeName System.Collections.ArrayList
        $bgRunspaceCounter    = 0
        $iss = [System.Management.Automation.Runspaces.InitialSessionState]::CreateDefault()

        $comps_in_job = 20

        $CompCol = gc $CompListPath
        if ($ThrottleLimit*$comps_in_job -gt $compCol.Length) { $array_num = $ThrottleLimit }
        else { [int]$array_num = [Math]::Ceiling($compCol.Length / $comps_in_job) }
        $tasks = @{}; $count = 0
        foreach ($PC in $compCol) {$tasks[$count % $array_num] += @($PC);$count++}
        [int]$Total = $tasks.Count
        
        foreach ($ExternalVariable in ('runspacetimers', 'ofs')) {
        $iss.Variables.Add((New-Object -TypeName System.Management.Automation.Runspaces.SessionStateVariableEntry -ArgumentList $ExternalVariable, (Get-Variable -Name $ExternalVariable -ValueOnly), ''))
        }
        
        $rp = [System.Management.Automation.Runspaces.RunspaceFactory]::CreateRunspacePool(1, $ThrottleLimit, $iss, $Host)
        $rp.ApartmentState = 'STA'
        $rp.Open()
 
        # This is the actual code called for each computer

        $ScriptBlock = {
            [CmdletBinding()]
            Param (
                [Parameter(Position=0)]
                $CompCol,
 
                [Parameter(Position=1)]
                [int]$bgRunspaceID
            )
            $runspacetimers.$bgRunspaceID = Get-Date
            $Result = @()
            
            foreach ($comp in $CompCol) {
            $WMIParam = @{'Computername'=$comp;'ErrorAction'='Stop';'Impersonation'='impersonate'}
                Try {
                $promisc=@() #(NDIS_PACKET_TYPE_PROMISCUOUS) --> detect network sniffers
                $wmi_promisc = gwmi @WMIParam -namespace "root\WMI" -query 'SELECT InstanceName,NdisCurrentPacketFilter FROM MSNdis_CurrentPacketFilter'
                foreach ($adp in $wmi_promisc) { if ($adp.NdisCurrentPacketFilter -band 0x00000020) {$promisc+=$adp.InstanceName} }

                $IP_Query = gwmi @WMIParam -Query "SELECT IPAddress,DHCPEnabled,MACAddress,Description FROM Win32_NetworkAdapterConfiguration WHERE IPEnabled='$True'"
                $IPAddress = "$(($IP_Query | Select-Object @{Name='IPAddress';Expression={[string]::join(",", ($_.IPAddress))}}).IPAddress)"

                $Compsys = gwmi @WMIParam -Query 'SELECT Manufacturer,Model,Name FROM win32_computersystem'

                $OS = gwmi @WMIParam -Query 'SELECT Caption,ServicePackMajorVersion,Version,TotalVisibleMemorySize,InstallDate,LastBootUpTime FROM Win32_OperatingSystem'

                $CPU = gwmi @WMIParam -Query 'SELECT AddressWidth,Name,MaxClockSpeed,NumberOfCores,NumberOfLogicalProcessors FROM Win32_Processor'

                $Disk = gwmi @WMIParam -Query "SELECT FreeSpace,Size,DeviceID FROM Win32_logicaldisk"

                $lastboot = [System.Management.ManagementDateTimeconverter]::ToDateTime("$($OS.LastBootUpTime)")
                $uptime = (Get-Date) - $lastboot

                <# VIRTUAL
                $bios = gwmi @WMIParam -Query 'Select Version,SerialNumber from Win32_BIOS'
                if ($bios.Version -match "VIRTUAL") { $Virtual = "Virtual - Hyper-V" }
                elseif ($bios.Version -match "A M I") { $Virtual = "Virtual - Virtual PC" }
                elseif ($bios.Version -like "*Xen*")  { $Virtual = "Virtual - Xen" }
                elseif ($bios.SerialNumber -like "*VMware*") { $Virtual = "Virtual - VMWare" }
                elseif ($Compsys.Manufacturer -like "*Microsoft*") { $Virtual = "Virtual - Hyper-V" }
                elseif ($Compsys.Manufacturer -like "*VMWare*") { $Virtual = "Virtual - VMWare" }
                elseif ($Compsys.Model -like "*Virtual*") { $Virtual = "Unknown Virtual Machine" }
                else { $Virtual = "Physical"} #>
                
                $Admins=@(); $GroupName=(gwmi @WMIParam -Query "Select name from win32_group where SID='S-1-5-32-544'").Name
                $AdminList=(gwmi @WMIParam -Query "SELECT PartComponent FROM Win32_GroupUser WHERE GroupComponent=`"Win32_Group.Domain='$comp',Name='$GroupName'`"").PartComponent
                foreach ($line in $AdminList) {$Admins += $line.Replace('Name=', '').Replace("`"",'').Replace(',','\').Split('=')[1]}

                $Result+=New-Object –TypeName PSCustomObject –Property ([ordered]@{
                    RealName = $Compsys.Name
                    DNSName = $comp
                    WMI = 'OK'
                    Manufacturer = $Compsys.Manufacturer
                    Model = $Compsys.Model
                    #Virtual = $Virtual
                    OS = $OS.Caption
                    Service_pack = $OS.ServicePackMajorVersion
                    OSVersion = $OS.Version
                    OS_bit = "x$($CPU.AddressWidth)"
                    Memory_MB = [int]($OS.TotalVisibleMemorySize/1KB)
                    CPU_Name = $CPU.Name
                    CPU_Speed = $CPU.MaxClockSpeed
                    CPU_Cores = $CPU.NumberOfCores
                    CPU_Logic_units = $CPU.NumberOfLogicalProcessors
                    C_Size = [int](($Disk | where DeviceID -eq 'C:').Size/1GB)
                    C_Free = "{0:N2}" -f (($Disk | where DeviceID -eq 'C:').FreeSpace/1GB)
                    All_disks_Size = [int]((($Disk.Size | Measure-Object -Sum).Sum)/1GB)
                    Installed = [System.Management.ManagementDateTimeconverter]::ToDateTime("$($OS.InstallDate)").ToShortDateString()
                    Lastboot = "$($lastboot.ToShortDateString()) $($lastboot.ToShortTimeString())"
                    Uptime = "$($uptime.Days) days, $($uptime.Hours) hours"
                    NIC_Description = "$($IP_Query.Description)"
                    IPAddress = "$IPAddress"
                    MACAddress = "$($IP_Query.MACAddress)"
                    DHCP = "$($IP_Query.DHCPEnabled)".Replace('True','DHCP').Replace('False','Static')
                    Promiscuous = "$promisc"
                    Admins = "$Admins"
                })
            } #End Try
            Catch {$Result+=New-Object –TypeName PSCustomObject –Property ([ordered]@{
                RealName = ''
                DNSName = $comp
                WMI = $_
                Manufacturer = ''
                Model = ''
                #Virtual = ''
                OS = ''
                Service_pack = ''
                OSVersion = ''
                OS_bit = ''
                Memory_MB = ''
                CPU_Name = ''
                CPU_Speed = ''
                CPU_Cores = ''
                CPU_Logic_units = ''
                C_Size = ''
                C_Free = ''
                All_disks_Size = ''
                Installed = ''
                Lastboot = ''
                Uptime = ''
                NIC_Description = ''
                IPAddress = ''
                MACAddress = ''
                DHCP = ''
                Promiscuous = ''
                Admins = ''
             })
        } #End try & catch 
    } #End foreach
    Write-Output -InputObject $Result
    } #End sb


  function Get-Result  {
            [CmdletBinding()]
            Param  ( [switch]$Wait )
            do {
                $More = $false
                foreach ($runspace in $runspaces) {
                    if ($runspace.Handle.isCompleted) {
                        $runspace.PowerShell.EndInvoke($runspace.Handle)
                        $runspace.PowerShell.Dispose()
                        $runspace.PowerShell = $null
                        $runspace.Handle = $null
                    }
                    elseif ($runspace.Handle -ne $null) { $More = $true }
                }
                if ($More -and $PSBoundParameters['Wait']) { Start-Sleep -Milliseconds 200 }
                foreach ($threat in $runspaces.Clone()) {
                    if ( -not $threat.handle) { $runspaces.Remove($threat) }
                }

                if ($ShowProgress) {
                    $ProgressSplatting = @{
                        Activity = 'Inventory: Getting info'
                        Status = 'Currently {0} of {1} total threads done' -f ($bgRunspaceCounter - $runspaces.Count), $bgRunspaceCounter
                        PercentComplete = ($bgRunspaceCounter - $runspaces.Count) / $bgRunspaceCounter * 100
                    }
                    Write-Progress @ProgressSplatting
                }
            }
            while ($More -and $PSBoundParameters['Wait'])
        }
    }
        
    PROCESS {
        0..($Total-1) | % {
            $bgRunspaceCounter++
            $psCMD = [System.Management.Automation.PowerShell]::Create().AddScript($ScriptBlock)
            $null = $psCMD.AddParameter('bgRunspaceID',$bgRunspaceCounter)
            $null = $psCMD.AddParameter('CompCol',$tasks[$_])
            $psCMD.RunspacePool = $rp
 
            [void]$runspaces.Add(@{
                Handle = $psCMD.BeginInvoke()
                PowerShell = $psCMD
                IObject = $tasks[$_]
                ID = $bgRunspaceCounter
           })
           Get-Result
        }
    }
    END {
        Get-Result -Wait
        $rp.Close()
        $rp.Dispose()
    }
}
Write-Host "Started at $(get-date)."
Get-Inventory | Export-Csv -Path "D:\Porocilo.csv" -Delimiter "`t" -encoding "unicode" -force -notypeinformation
Write-Host "Ended at $(get-date)."